<?php
/*
 * BSMTP.php - provide BSMTP functions
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU Lesser General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Mail_BSMTP
 */

class Mail_BSMTP {

	/**
	 * Whether or not to keep the BSMTP temporary queue files after calling sendQueue()
	 *
	 * @var bool
	 */
	public $keep_temp_files_on_send = false;
	
	protected $_tmp_file_handle;
	protected $_tmp_file;

	/**
	 * The driver object, if loaded
	 *
	 * @var Mail_BSMTP_Driver_Interface
	 */
	protected $_driver = false;
	
	/**
	 * Constructor
	 *
	 * @param string $tmp_dir  The temporary directory to store BSMTP files in
	 */
	public function __construct($tmp_dir = '')
	{
		$this->_initQueue($tmp_dir);
	}

	/**
	 * Initialise the BSMTP queue by creating a temp file
	 *
	 * @param string $tmp_dir  The temporary directory to use for BSMTP queue files. Uses system temp dir if not specified.
	 */
	protected function _initQueue($tmp_dir = '')
	{
		$this->_tmp_file = tempnam($tmp_dir, 'Mail_BSMTP');
		if (!$this->_tmp_file_handle = fopen($this->_tmp_file, 'w')) {
			throw new Exception("Could not open BSMTP temp file $this->_tmp_file");
		}
	}

	/**
	 * Load a BSMTP driver, which enables the actual sending of the mail queue.
	 *
	 * @param string $driver  The name of an installed driver. Case-sensitive.
	 */
	public function loadDriver($driver)
	{
		require_once 'Mail' . DIRECTORY_SEPARATOR . 'BSMTP' . DIRECTORY_SEPARATOR . 'Driver' . DIRECTORY_SEPARATOR . $driver . '.php';
		$driver_class = "Mail_BSMTP_Driver_$driver";
		$this->_driver = new $driver_class;
		
		if (!$this->_driver instanceof Mail_BSMTP_Driver) {
			throw new Exception('Loaded driver is not valid');
		}
	}
	
	/**
	 * Add a mail to the BSMTP queue
	 *
	 * @param string $to         Recipient of the mail
	 * @param string $subject    Subject line
	 * @param string $message    Message body
	 * @param string $env_sender Envelope sender of the mail
	 * @param array $headers     Additional headers to add to the mail, in the format headername => value
	 */
	public function addMail($to, $subject, $message, $env_sender, $headers=array())
	{
		$headers['Subject'] = $subject;
		
		$headers_tmp = array_change_key_case($headers);
		if (!isset($headers_tmp['date'])) {
			$headers['Date'] = date('r');
		}
		
		$smtp_message = '';
		foreach ($headers as $name => $val) {
			$smtp_message .= "$name: $val\n";
		}
		$smtp_message .= "\n";
		
		$message_lines= explode("\n", $message);
		foreach ($message_lines as $line) {
			// Escape lines starting with periods as per RFC2821, to avoid truncated or corrupt messages
			if (substr($line, 0, 1) == '.') {
					$line = '.' . $line;
			}
			$smtp_message .= $line . "\n";
		}
		
		$this->addRawMail($to, $smtp_message, $env_sender);
	}
	
	/**
	 * Add a raw SMTP message to the BSMTP queue
	 *
	 * @param string|array $to    Recipient(s) of the mail. Use array if more than one. Must be valid e-mail addresses
	 * @param string $message     The raw message including headers
	 * @param string $env_sender  Envelope sender of the mail
	 */
	public function addRawMail($to, $message, $env_sender)
	{
		if (!is_array($to)) {
			$to = array($to);
		}
		
		$final_recipients = array();
		$rejected_recipients = array();
		foreach ($to as $recipient) {
			$recipient = trim($recipient);
			if (!empty($recipient)) {
				if (preg_match('/^.+@.+\..+$/', $recipient)) {
					if ($this->_driver !== false) {
						if ($this->_driver->validateAddress($recipient)) {
							$final_recipients[] = $recipient;
						} else {
							$rejected_recipients[] = $recipient;
						}
					} else {
						$final_recipients[] = $recipient;
					}
				} else {
					$rejected_recipients[] = $recipient;
				}
			}
		}
		
		if (count($final_recipients) > 0) {
			fputs($this->_tmp_file_handle, "MAIL FROM:<$env_sender>\n");
			foreach ($final_recipients as $recipient) {
				fputs($this->_tmp_file_handle, "RCPT TO:<$recipient>\n");
			}
			fputs($this->_tmp_file_handle, "DATA\n");
			fputs($this->_tmp_file_handle, "$message\n");
			fputs($this->_tmp_file_handle, ".\n");
		}
		
		if (count($rejected_recipients) > 0) {
			throw new Exception('The following recipients were rejected due to bad syntax or driver rejection: ' . implode(',', $rejected_recipients));
		}
	}
	
	/**
	 * Get the filename of the file containing the BSMTP output
	 *
	 * @return string Full filesystem path to BSMTP file
	 */
	public function getBsmtpFile()
	{
		return $this->_tmp_file;
	}
	
	/**
	 * Send the queued mail using the loaded driver
	 * 
	 * @see loadDriver()
	 *
	 */
	public function sendQueue()
	{
		if ($this->_driver == false) {
			throw new Exception('You must load a driver before calling sendQueue()');
		}
		
		fclose($this->_tmp_file_handle);
		
		$this->_driver->sendQueue($this->_tmp_file);
	
		if (!$this->keep_temp_files_on_send) {
			unlink($this->_tmp_file);
		}
	}
}
